@@ -373,6 +373,41 @@ static void testMaximizeSize() {
     EXPECT(Tests::windowCount(), 0);
 }
 
+static void testGroupFallbackFocus() {
+    NLog::log("{}Testing group fallback focus", Colors::GREEN);
+
+    EXPECT(spawnKitty("kitty_A"), true);
+
+    OK(getFromSocket("/dispatch togglegroup"));
+
+    EXPECT(spawnKitty("kitty_B"), true);
+    EXPECT(spawnKitty("kitty_C"), true);
+    EXPECT(spawnKitty("kitty_D"), true);
+
+    {
+        auto str = getFromSocket("/activewindow");
+        EXPECT(str.contains("class: kitty_D"), true);
+    }
+
+    OK(getFromSocket("/dispatch focuswindow class:kitty_B"));
+    OK(getFromSocket("/dispatch focuswindow class:kitty_D"));
+    OK(getFromSocket("/dispatch killactive"));
+
+    Tests::waitUntilWindowsN(3);
+
+    // Focus must return to the last focus, in this case B.
+    {
+        auto str = getFromSocket("/activewindow");
+        EXPECT(str.contains("class: kitty_B"), true);
+    }
+
+    NLog::log("{}Killing all windows", Colors::YELLOW);
+    Tests::killAllWindows();
+
+    NLog::log("{}Expecting 0 windows", Colors::YELLOW);
+    EXPECT(Tests::windowCount(), 0);
+}
+
 static void testBringActiveToTopMouseMovement() {
     NLog::log("{}Testing bringactivetotop mouse movement", Colors::GREEN);
 
@@ -412,6 +447,39 @@ static void testBringActiveToTopMouseMovement() {
     Tests::killAllWindows();
 }
 
+static void testInitialFloatSize() {
+    NLog::log("{}Testing initial float size", Colors::GREEN);
+
+    Tests::killAllWindows();
+    OK(getFromSocket("/keyword windowrule match:class kitty, float yes"));
+    OK(getFromSocket("/keyword input:float_switch_override_focus 0"));
+
+    EXPECT(spawnKitty("kitty"), true);
+
+    {
+        // Kitty by default opens as 640x400, if this changes this test will break
+        auto str = getFromSocket("/clients");
+        EXPECT(str.contains("size: 640,400"), true);
+    }
+
+    OK(getFromSocket("/reload"));
+
+    Tests::killAllWindows();
+
+    OK(getFromSocket("/dispatch exec [float yes]kitty"));
+
+    Tests::waitUntilWindowsN(1);
+
+    {
+        // Kitty by default opens as 640x400, if this changes this test will break
+        auto str = getFromSocket("/clients");
+        EXPECT(str.contains("size: 640,400"), true);
+        EXPECT(str.contains("floating: 1"), true);
+    }
+
+    Tests::killAllWindows();
+}
+
 static bool test() {
     NLog::log("{}Testing windows", Colors::GREEN);
 
@@ -842,10 +910,10 @@ static bool test() {
     Tests::killAllWindows();
 
     testGroupRules();
-
     testMaximizeSize();
-
     testBringActiveToTopMouseMovement();
+    testGroupFallbackFocus();
+    testInitialFloatSize();
 
     NLog::log("{}Reloading config", Colors::YELLOW);
     OK(getFromSocket("/reload"));
